
#include <string.h>
#include "bsp.h"
#include "mrfi.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "nwk_frame.h"
#include "nwk.h"
#include "virtual_com_cmds.h"

#include "app_remap_led.h"


//Prototypes
void toggleLED(uint8_t);
void startTimerA();
void stopTimerA();
void sendBeacon();
static void processMessage(linkID_t, uint8_t *, uint8_t);
void processValidPacket(linkID_t, uint8_t *,uint8_t);
void processNonValidPacket(linkID_t, uint8_t *,uint8_t);
void givePointTo(uint8_t);
void giveNonValid(uint8_t);
void processDoubleTouch();
void stopReceiving();
void startReceiving();

/* callback handler */
static uint8_t sCB(linkID_t);



/* reserve space for the maximum possible peer Link IDs */
static uint8_t NUM_MAX_CONNECTIONS = 2;
static uint8_t state = 0;

static linkID_t sLID[2] = {0,0};
static uint8_t  sNumCurrentPeers = 0;


//struct where the fencers will be stored
typedef struct{
	linkID_t link;
	uint8_t colour;
	uint8_t *msg_rcv;
}Fencer;

static Fencer fencers[2];


/* work loop semaphores */
static volatile uint8_t sPeerFrameSem = 0;
static volatile uint8_t sJoinSem = 0;


#define SPIN_ABOUT_A_QUARTER_SECOND   NWK_DELAY(250)

//States
#define JOINING 0
#define WAITING 1
#define ONE_TOUCH 2
#define ZERO_TOUCH 3
#define ONE_NON_VALID 4
#define ZERO_NON_VALID 5
#define RESET 6

//Fencers colours
#define GREEN 0
#define RED 1

#define VALID 1
#define NONVALID 2


#define MILISEC_DOUBLETOUCH 40				//from 1 to 4000 ms
#define CICLES_DOUBLETOUCH 12*MILISEC_DOUBLETOUCH    


void main (void)
{
  bspIState_t intState;
  
  WDTCTL = WDTPW|WDTHOLD;
  
  BSP_Init();
  
  COM_Init();
  
  
  __enable_interrupt();
  


  SMPL_Init(sCB);

  /* green and red LEDs on solid to indicate waiting for a Join. */
  if (!BSP_LED2_IS_ON()) toggleLED(RED);
  
  if (!BSP_LED1_IS_ON()) toggleLED(GREEN);
  
  
  state = JOINING;

  /* main work loop */
  while (sNumCurrentPeers<NUM_MAX_CONNECTIONS)
  {
    /* Wait for the Join semaphore to be set by the receipt of a Join frame from a
     * device that supports an End Device.
     */
    if (sJoinSem && (sNumCurrentPeers < NUM_MAX_CONNECTIONS))
    {
      /* listen for a new connection */
      while (1)
      {
        if (SMPL_SUCCESS == SMPL_LinkListen(&sLID[sNumCurrentPeers]))
        {
          fencers[sNumCurrentPeers].link=sLID[sNumCurrentPeers];
          fencers[sNumCurrentPeers].colour=sNumCurrentPeers;
          break;
        }
        /* Implement fail-to-link policy here. otherwise, listen again. */
      }

      sNumCurrentPeers++;

      BSP_ENTER_CRITICAL_SECTION(intState);
      sJoinSem--;
      BSP_EXIT_CRITICAL_SECTION(intState);
    }
  }
  
  if (BSP_LED2_IS_ON()) toggleLED(RED);

  if (BSP_LED1_IS_ON()) toggleLED(GREEN);

  
  
  SPIN_ABOUT_A_QUARTER_SECOND;
  SPIN_ABOUT_A_QUARTER_SECOND;
  
  sendBeacon();
  
  state = WAITING;
  
  
  while(1){

    /* Have we received a frame on one of the ED connections?
     * No critical section -- it doesn't really matter much if we miss a poll
     */
     
    if(state==RESET) {
    	
    	NWK_DELAY(1000);
    	startReceiving();   	
    	sendBeacon();
    }
     
    if (sPeerFrameSem)
    {
      uint8_t     msg[5], len, i;

      /* process all frames waiting */
      for (i=0; i<sNumCurrentPeers; ++i)
      {
        if (SMPL_SUCCESS == SMPL_Receive(sLID[i], msg, &len))
        {
        	
          if(sLID[i]==fencers[GREEN].link){
          		fencers[GREEN].msg_rcv=msg;
          }
          
          else if(sLID[i]==fencers[RED].link){
          		fencers[RED].msg_rcv=msg;
          }
          	
       
          processMessage(sLID[i], msg, len);
		  
		  if(sPeerFrameSem==1){
          	BSP_ENTER_CRITICAL_SECTION(intState);
          	sPeerFrameSem--;
          	BSP_EXIT_CRITICAL_SECTION(intState);
          	break;
		  }
        }
      }
    }					    
  }
}

void toggleLED(uint8_t which)
{
  if (0 == which) BSP_TOGGLE_LED1();
  
  else if (1 == which) BSP_TOGGLE_LED2();

  return;
}

/* Runs in ISR context. Reading the frame should be done in the */
/* application thread not in the ISR thread. */
static uint8_t sCB(linkID_t lid)
{
  //uint8_t msg[5], len;
	
  if (state == JOINING )
  {
  	sJoinSem++;
  }
  
  
  else if (state==RESET){
  }
  
  else
  {
  	sendBeacon();
    //sPeerFrameSem++;
  }

  
  return 0;
}

static void processMessage(linkID_t lid, uint8_t *msg, uint8_t len)
{
  uint8_t type=msg[0];
  
  /* do something useful */
  if (state==WAITING)
  {
  	
  	if(type==VALID) processValidPacket(lid,msg,len);
  	else if (type==NONVALID) processNonValidPacket(lid,msg,len);
  
  }
  else if (state == ONE_TOUCH){
  	/*stopTimerA();
  	state=RESET;*/
  	stopReceiving();
    if(type==VALID) {
    	if(lid==fencers[1].link){
          fencers[1].msg_rcv=msg;
          processDoubleTouch();
    	}
    }
    	 
    else if(type==NONVALID) givePointTo(1);
  }
  
  else if (state == ZERO_TOUCH){
  	/*stopTimerA();
  	state=RESET;*/
  	stopReceiving();
    if(type==VALID) {
    	if(lid==fencers[0].link){
          fencers[0].msg_rcv=msg;
          processDoubleTouch();
    	}
    }
    	 
    else if(type==NONVALID) givePointTo(0);
        
  }
  
  else if(state == ONE_NON_VALID){
  		/*stopTimerA();
  	state=RESET;*/
  	stopReceiving();
  		if(type==NONVALID) {
  			if(lid == fencers[0].link) giveNonValid(3); 
  		}
        else if(type==VALID) {
        	if(lid == fencers[1].link) givePointTo(0);
        }
  		
  	
  }
  else if(state == ZERO_NON_VALID){
  		/*stopTimerA();
  	state=RESET;*/
  	stopReceiving();
  		if(type==NONVALID) {
  			if(lid == fencers[1].link) giveNonValid(3); 
  		}
        else if(type==VALID) {
        	if(lid == fencers[0].link) givePointTo(1);
        } 		
  	
  }
  
  
  
  return;
}

void processValidPacket(linkID_t lid, uint8_t *msg,uint8_t len){
	//uint8_t *msg_rcv=msg;
	if(lid==fencers[0].link) state=ONE_TOUCH;
	else if(lid==fencers[1].link) state=ZERO_TOUCH;
	startTimerA();
}

void processNonValidPacket(linkID_t lid, uint8_t *msg,uint8_t len){
	//uint8_t *msg_rcv=msg;
	if(lid==fencers[0].link) state=ZERO_NON_VALID;
	else if(lid==fencers[1].link) state=ONE_NON_VALID;
	startTimerA();
	
}

void processDoubleTouch(){
	
	uint8_t *green_time=fencers[GREEN].msg_rcv;
	uint8_t *red_time=fencers[RED].msg_rcv;
	/*
	if(green_time[2]<red_time[2]) givePointTo(GREEN);
	else if(green_time[2]>red_time[2]) givePointTo(RED);
	else{
		if(green_time[3]<red_time[3]) givePointTo(GREEN);
		else if(green_time[3]>red_time[3]) givePointTo(RED);
		else{
			if(green_time[4]<red_time[4]) givePointTo(GREEN);
			else if(green_time[4]>red_time[4]) givePointTo(RED);
		}
	}*/
	
	uint8_t green_cicles=(green_time[2]*60*255)+(green_time[3]*255)+(green_time[4]);
	uint8_t red_cicles=(red_time[2]*60*255)+(red_time[3]*255)+(red_time[4]);
	
	if (green_cicles<red_cicles){
		if ((red_cicles-green_cicles)<40) givePointTo(3);
		else givePointTo(GREEN);
	}
	else if (green_cicles>red_cicles){
		if ((green_cicles-red_cicles)<40) givePointTo(3);
		else givePointTo(RED);
	}
	
		
	
}

							
void sendBeacon(){
      uint8_t i=0;
      
      /*
      for(i=0;i<sNumCurrentPeers;i++){
      	if(SMPL_SUCCESS==SMPL_Send(sLID[i], &i, sizeof(i))){
      		//break;
      	}
	  }
      */
      
      SMPL_Send(SMPL_LINKID_USER_UUD, &i, sizeof(i));
      
      
      	
      //SMPL_Send(sLID[1], 0, 0);
   
}

void givePointTo(uint8_t fencer){
	
    /*stopTimerA();
  	state=RESET;*/
  	stopReceiving();

	
	if(fencer==0){
			TXString( "Punto1\n", 7 );
			BSP_TURN_ON_LED1();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			
	}
	if(fencer==1){
			TXString( "Punto2\n", 7 );
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED2();
			
	}
	
	if(fencer==3){
			TXString( "Punto3\n", 7 );
			BSP_TURN_ON_LED1();
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			BSP_TURN_OFF_LED2();
			
	}
	
	
	//state=RESET;
	//TACTL |= MC_0;
	//TACCTL0 ^= ~CCIE;
	//BSP_DISABLE_INTERRUPTS();
	
				
}

void giveNonValid(uint8_t fencer){
	
	/*stopTimerA();
  	state=RESET;*/
  	stopReceiving();
	
	if(fencer==0){
			TXString("Non1\n", 5);
			BSP_TURN_ON_LED1();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			
	}
	else if(fencer==1){
			TXString("Non2\n", 5);
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED2();
			
	}
	else if(fencer==3){
			TXString("Non3\n", 5);
			BSP_TURN_ON_LED1();
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			BSP_TURN_OFF_LED2();
	}
	
	//state=RESET;
	//TACTL |= MC_0;
	//TACCTL0 ^= ~CCIE;
	//BSP_DISABLE_INTERRUPTS();
	
				
}

void stopTimerA(){
	TACCTL0 &= ~CCIE; 	
}

void startReceiving(){
	SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_RXON, 0);
	//stopTimerA();
	state=WAITING;
}

void stopReceiving(){
	SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_RXIDLE, 0);
	stopTimerA();
	state=RESET;
}

void startTimerA(){
	
	/*
	 * TACCR0 = 49999; // Upper limit of count for TAR
	TACCTL0 = CCIE; // Enable interrupts on Compare 0
	TACTL = MC_3|ID_3|TASSEL_2|TACLR; // Set up and start Timer A
	// "Up to CCR0" mode , divide clock by 8, clock from SMCLK , clear timer
	*/
	
	BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
  	TACCTL0 = CCIE;                           // TACCR0 interrupt enabled
  	TACCR0 = CICLES_DOUBLETOUCH;                           // ~ 1 sec
  	TACTL = TASSEL_1 | MC_1 | TACLR;                  // ACLK, upmode
	
	//__enable_interrupt();
}



#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void) // ISR for TACCR1 CCIFG and TAIFG
{
	if(state==ONE_TOUCH){
		givePointTo(1);
	}
	else if(state==ZERO_TOUCH){
		givePointTo(0);
	}
	else if(state==ONE_NON_VALID){
		giveNonValid(1);
	}
	else if(state==ZERO_NON_VALID){
		giveNonValid(0);
	}
	
	
	
			
}

//#pragma vector = PORT2_VECTOR
//__interrupt void PORT2_ISR (void)
BSP_ISR_FUNCTION( BSP_GpioPort1Isr, PORT2_VECTOR )
{
  //
   // This ISR is easily replaced.  The new ISR must simply
   //  include the following function call.
   //

		
   
  MRFI_GpioIsr();
}






