
#include <string.h>
#include "bsp.h"
#include "mrfi.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "nwk_frame.h"
#include "nwk.h"
#include "virtual_com_cmds.h"

#include "app_remap_led.h"


//Prototypes
void toggleLED(uint8_t);
void startTimerA();
void stopTimerA();
void startTimerB();
void stopTimerB();
void sendBeacon();
static void processMessage(linkID_t, uint8_t *, uint8_t);
void processValidPacket(linkID_t);
void processNonValidPacket(linkID_t);
void givePointTo(uint8_t);
void giveNonValid(uint8_t);
void processDoubleTouch();
void stopReceiving();
void startReceiving();

/* callback handler */
static uint8_t sCB(linkID_t);



/* reserve space for the maximum possible peer Link IDs */
static uint8_t NUM_MAX_CONNECTIONS = 2;
static uint8_t state = 0;


static uint8_t errors = 0;
static uint8_t beacons = 0;
static uint8_t beacons2 = 0;
static uint8_t msgs = 0;

static linkID_t sLID[2] = {0,0};
static linkID_t lidAux=0;

static uint8_t  sNumCurrentPeers = 0;


//struct where the fencers will be stored
typedef struct{
	linkID_t link;
	uint8_t msg_rcv[4];
}Fencer;

static Fencer fencers[2];


/* work loop semaphores */
static volatile uint8_t sPeerFrameSem = 0;
static volatile uint8_t sJoinSem = 0;


#define SPIN_ABOUT_A_QUARTER_SECOND   NWK_DELAY(250)

//States
#define JOINING 0
#define WAITING 1
#define ONE_TOUCH 2
#define ZERO_TOUCH 3
#define ONE_NON_VALID 4
#define ZERO_NON_VALID 5
#define RESET 6

//Fencers colours
#define GREEN 0
#define RED 1

#define VALID 1
#define NONVALID 2


#define MILISEC_DOUBLETOUCH 1000				//from 1 to 4000 ms
#define CICLES_DOUBLETOUCH 12*MILISEC_DOUBLETOUCH   

#define MILISEC_TIC 50				//from 1 to 4000 ms
#define CICLES_TIC 12*MILISEC_TIC


void main (void)
{
  bspIState_t intState;
  
  BSP_Init();
  BSP_ENABLE_INTERRUPTS();
  
  P2DIR |= BIT3;
  
  COM_Init();
  
  SMPL_Init(sCB);

  /* green and red LEDs on solid to indicate waiting for a Join. */
  if (!BSP_LED2_IS_ON()) toggleLED(RED);
  
  if (!BSP_LED1_IS_ON()) toggleLED(GREEN);
  
  
  state = JOINING;

  /* main work loop */
  while (sNumCurrentPeers<NUM_MAX_CONNECTIONS)
  {
    /* Wait for the Join semaphore to be set by the receipt of a Join frame from a
     * device that supports an End Device.
     */
    if (sJoinSem && (sNumCurrentPeers < NUM_MAX_CONNECTIONS))
    {
      /* listen for a new connection */
      while (1)
      {
        if (SMPL_SUCCESS == SMPL_LinkListen(&sLID[sNumCurrentPeers]))
        {
          fencers[sNumCurrentPeers].link=sLID[sNumCurrentPeers];
          //fencers[sNumCurrentPeers].colour=sNumCurrentPeers;
          break;
        }
        /* Implement fail-to-link policy here. otherwise, listen again. */
      }

      sNumCurrentPeers++;

      BSP_ENTER_CRITICAL_SECTION(intState);
      sJoinSem--;
      BSP_EXIT_CRITICAL_SECTION(intState);
    }
  }
  
  if (BSP_LED2_IS_ON()) toggleLED(RED);

  if (BSP_LED1_IS_ON()) toggleLED(GREEN);

  
  
  NWK_DELAY(500);
  
  sendBeacon(3);
  startTimerB();
  
  state = WAITING;
  
  
  while(1){

    /* Have we received a frame on one of the ED connections?
     * No critical section -- it doesn't really matter much if we miss a poll
     */
     
    if(state==RESET) {
    	
    	NWK_DELAY(1000);
    	startReceiving();   	
    	sendBeacon(3);
    	startTimerB();
    	state=WAITING;
    }
    
    if(state==ONE_TOUCH) {
    	sendBeacon(1);
    	
    }
    
    if(state==ZERO_TOUCH) {
    	sendBeacon(0);
    	
    }
    	
    
     
    if (sPeerFrameSem)
    {
      uint8_t     msg[4], len, i;

     
     
        if (SMPL_SUCCESS == SMPL_Receive(lidAux, msg, &len))
        {
          //msgs=i+1;
          if(lidAux==fencers[GREEN].link){
          		fencers[GREEN].msg_rcv[0]=msg[0];
          		fencers[GREEN].msg_rcv[1]=msg[1];
          		fencers[GREEN].msg_rcv[2]=msg[2];
          		fencers[GREEN].msg_rcv[3]=msg[3];
          }
          
          else if(lidAux==fencers[RED].link){
          		fencers[RED].msg_rcv[0]=msg[0];
          		fencers[RED].msg_rcv[1]=msg[1];
          		fencers[RED].msg_rcv[2]=msg[2];
          		fencers[RED].msg_rcv[3]=msg[3];
          }
          	
       
          processMessage(sLID[i], msg, len);
          
          BSP_ENTER_CRITICAL_SECTION(intState);
          sPeerFrameSem--;
          BSP_EXIT_CRITICAL_SECTION(intState);
		  
		 
		  	
		  	
        }
        /*if(sPeerFrameSem==0){
          	
          	break;
		}*/
      }
    }					    
  }
}

void toggleLED(uint8_t which)
{
  if (0 == which) BSP_TOGGLE_LED1();
  
  else if (1 == which) BSP_TOGGLE_LED2();

  return;
}

/* Runs in ISR context. Reading the frame should be done in the */
/* application thread not in the ISR thread. */
static uint8_t sCB(linkID_t lid)
{
  //uint8_t msg[5], len;
	
  if (state == JOINING )
  {
  	sJoinSem++;
  }
  
  
  else if (state==RESET){
  }
  
  else
  {
  	//sendBeacon();
    sPeerFrameSem++;
  }

  
  return 0;
}

static void processMessage(linkID_t lid, uint8_t *msg, uint8_t len)
{
  uint8_t type=msg[0];
  
  /* do something useful */
  if (state==WAITING)
  {
  	
  	if(type==VALID){
  		processValidPacket(lid);
  		
  	else if (type==NONVALID) {
  		if(lid==fencers[1].link) giveNonValid(1);
  		else if(lid==fencers[0].link) giveNonValid(0);
  	}
  
  }
  else if (state == ONE_TOUCH){
  	stopReceiving();
    if(type==VALID) {
    	if(lid==fencers[1].link){
          processDoubleTouch();
    	}
    }
    	 
    else if(type==NONVALID) givePointTo(1);
  }
  
  else if (state == ZERO_TOUCH){

  	stopReceiving();
    if(type==VALID) {
    	if(lid==fencers[0].link){
          processDoubleTouch();
    	}
    }
    	 
    else if(type==NONVALID) givePointTo(0);
        
  }
/*  
  else if(state == ONE_NON_VALID){
  	
  	stopReceiving();
  	if(type==NONVALID) {
  		if(lid == fencers[0].link) giveNonValid(3); 
  	}
    else if(type==VALID) {
        if(lid == fencers[1].link) givePointTo(0);
    }
  		
  	
  }
  else if(state == ZERO_NON_VALID){
  		
  	stopReceiving();
  	if(type==NONVALID) {
  		if(lid == fencers[1].link) giveNonValid(3); 
  	}
    else if(type==VALID) {
        if(lid == fencers[0].link) givePointTo(1);
    } 		
  	
  }
*/  
  
  
  return;
}

void processValidPacket(linkID_t lid){
	//uint8_t *msg_rcv=msg;
	if(lid==fencers[0].link) state=ONE_TOUCH;
	else if(lid==fencers[1].link) state=ZERO_TOUCH;
	//startTimerA();
}
/*
void processNonValidPacket(linkID_t lid){
	//uint8_t *msg_rcv=msg;
	if(lid==fencers[0].link) state=ZERO_NON_VALID;
	else if(lid==fencers[1].link) state=ONE_NON_VALID;
	startTimerA();
	
}
*/
void processDoubleTouch(){
	
	uint8_t *green_time=fencers[GREEN].msg_rcv;
	uint8_t *red_time=fencers[RED].msg_rcv;
	
	/*
	uint8_t green_cicles=(green_time[2]*60*255)+(green_time[3]*255)+(green_time[4]);
	uint8_t red_cicles=(red_time[2]*60*255)+(red_time[3]*255)+(red_time[4]);
	
	if (green_cicles<red_cicles){
		if ((red_cicles-green_cicles)<40) givePointTo(3);
		else givePointTo(GREEN);
	}
	else if (green_cicles>red_cicles){
		if ((green_cicles-red_cicles)<40) givePointTo(3);
		else givePointTo(RED);
	}*/
	
	if(green_time[1]>red_time[1]) givePointTo(RED);
	else if (green_time[1]<red_time[1]) givePointTo(GREEN);
	else{
		if(green_time[2]>red_time[2]) givePointTo(RED);
		else if (green_time[2]<red_time[2]) givePointTo(GREEN);
		else{
			if(green_time[3]>red_time[3]){
				if ((green_time[3]-red_time[3])>1)givePointTo(RED);
				else givePointTo(3);
			}
			else if (green_time[3]<red_time[3]){
				if ((red_time[3]-green_time[3])>1)givePointTo(GREEN);
				else givePointTo(3);
			}
			else{
				givePointTo(3);
			}
		}
		
	}
	
		
	
}

							
void sendBeacon(uint8_t fencer){
      uint8_t i=0;
      linkID_t j=0;
      
      if(state==RESET) i=1;     
      if(fencer==1) j=2;
      if(fencer==0) j=1;
      if(fencer==3) j=SMPL_LINKID_USER_UUD;
      
      while (SMPL_SUCCESS != SMPL_Send(j, &i, sizeof(i))){
      	errors++;
      }
      beacons++;
      if(beacons==0) beacons2++;
      P2OUT ^= BIT3;
}

void givePointTo(uint8_t fencer){
	
    /*stopTimerA();
  	state=RESET;*/
  	stopReceiving();

	
	if(fencer==0){
			TXString( "Punto1\n", 7 );
			BSP_TURN_ON_LED1();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			
	}
	if(fencer==1){
			TXString( "Punto2\n", 7 );
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED2();
			
	}
	
	if(fencer==3){
			TXString( "Punto3\n", 7 );
			BSP_TURN_ON_LED1();
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			BSP_TURN_OFF_LED2();
			
	}
	
	
	//state=RESET;
	//TACTL |= MC_0;
	//TACCTL0 ^= ~CCIE;
	//BSP_DISABLE_INTERRUPTS();
	
				
}

void giveNonValid(uint8_t fencer){
	
	/*stopTimerA();
  	state=RESET;*/
  	stopReceiving();
	
	if(fencer==0){
			TXString("Non1\n", 5);
			BSP_TURN_ON_LED1();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			
	}
	else if(fencer==1){
			TXString("Non2\n", 5);
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED2();
			
	}
	else if(fencer==3){
			TXString("Non3\n", 5);
			BSP_TURN_ON_LED1();
			BSP_TURN_ON_LED2();
			NWK_DELAY(500);
			BSP_TURN_OFF_LED1();
			BSP_TURN_OFF_LED2();
	}
	
	//state=RESET;
	//TACTL |= MC_0;
	//TACCTL0 ^= ~CCIE;
	//BSP_DISABLE_INTERRUPTS();
	
				
}

void stopTimerA(){
	TACCTL0 &= ~CCIE; 	
}

/*void stopTimerB(){
	TBCCTL0 &= ~CCIE; 	
}*/

void startReceiving(){
	SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_RXON, 0);
	//stopTimerA();
	
}

void stopReceiving(){
	SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_RXIDLE, 0);
	stopTimerA();
	state=RESET;
}

void startTimerA(){
	/*
	TACCR0 = CICLES_DOUBLETOUCH; // Upper limit of count for TAR
	TACCTL0 = CCIE; // Enable interrupts on Compare 0
	TACTL = MC_1|ID_3|TASSEL_2|TACLR; // Set up and start Timer A
	*/
	BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
  	TACCTL0 = CCIE;                           // TACCR0 interrupt enabled
  	TACCR0 = CICLES_DOUBLETOUCH;                           // ~ 1 sec
  	TACTL = TASSEL_1 | MC_1 | TACLR;                  // ACLK, upmode
	
	//__enable_interrupt();
}


void startTimerB(){
	/*
	TBCCR0 = CICLES_TIC; // Upper limit of count for TAR
	TBCCTL0 = CCIE; // Enable interrupts on Compare 0
	TBCTL = MC_1|ID_3|TBSSEL_2|TBCLR; // Set up and start Timer A
	*/
	BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
  	TBCCTL0 = CCIE;                           // TACCR0 interrupt enabled
  	TBCCR0 = CICLES_TIC;                        // ~ 1 sec
  	TBCTL = TBSSEL_1 | MC_1 | TBCLR;                  // ACLK, upmode
	
	//__enable_interrupt();
}




#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void) // ISR for TACCR1 CCIFG and TAIFG
{
	if(state==ONE_TOUCH){
		if (msgs==1) givePointTo(1);
		else if (msgs==2) processDoubleTouch();
	}
	else if(state==ZERO_TOUCH){
		if (msgs==1) givePointTo(0);
		else if (msgs==2) processDoubleTouch();
	}
	else if(state==ONE_NON_VALID){
		giveNonValid(1);
	}
	else if(state==ZERO_NON_VALID){
		giveNonValid(0);
	}
	
	
	
			
}

#pragma vector = TIMERB0_VECTOR
__interrupt void TIMERB0_ISR (void) // ISR for TACCR1 CCIFG and TAIFG
{
	sendBeacon();
			
}

//#pragma vector = PORT2_VECTOR
//__interrupt void PORT2_ISR (void)
BSP_ISR_FUNCTION( BSP_GpioPort1Isr, PORT2_VECTOR )
{
  //
   // This ISR is easily replaced.  The new ISR must simply
   //  include the following function call.
   //

		
   
  MRFI_GpioIsr();
}




